@@ -1,7 +1,7 @@
-/* $Id: wmwork.c,v 1.64 2003/04/29 04:24:04 godisch Exp $
+/* $Id: wmwork.c,v 1.67 2005/12/02 07:36:59 godisch Exp $
  * vim: set noet ts=4:
  *
- * Copyright (c) 2002-2003 Martin A. Godisch <martin@godisch.de>
+ * Copyright (c) 2002-2005 Martin A. Godisch <martin@godisch.de>
  */
 
 #include <assert.h>
@@ -9,17 +9,25 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <getopt.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <unistd.h>
 #include <time.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <X11/xpm.h>
 
+#ifdef SOLARIS
+#  include <strings.h>
+#  define PID_T "%lu"
+#else  /* SOLARIS */
+#  include <getopt.h>
+#  define PID_T "%u"
+#endif /* SOLARIS */
+
 #include "wmgeneral.h"
 #include "wmwork.h"
 #include "wmwork.xpm"
@@ -41,7 +49,7 @@
 
 char
 	wmwork_mask_bits[64*64],
-	*dirname  = NULL,
+	*dirName  = NULL,
 	*logname  = NULL,
 	*lockname = NULL;
 struct timeval
@@ -95,24 +103,24 @@ int main(int argc, char *argv[])
 	do_opts(argc, argv, &show_days, &xdisplay, &geometry, &force);
 
 	path_len = strlen(getenv("HOME")) + strlen("/.wmwork/worklog") + 1;
-	if ((dirname = malloc(path_len)) == NULL || (logname = malloc(path_len)) == NULL || (lockname = malloc(path_len)) == NULL) {
+	if ((dirName = malloc(path_len)) == NULL || (logname = malloc(path_len)) == NULL || (lockname = malloc(path_len)) == NULL) {
 		fprintf(stderr, "%s: cannot allocate memory for path variable\n", PACKAGE_NAME);
 		fprintf(stderr, "%s: %s\n", PACKAGE_NAME, strerror(errno));
 		exit(1);
 	}
-	snprintf(dirname,  path_len, "%s/.wmwork", getenv("HOME"));
-	snprintf(logname,  path_len, "%s/worklog", dirname);
-	snprintf(lockname, path_len, "%s/.#LOCK",  dirname);
-	if (chdir(dirname) < 0) {
+	snprintf(dirName,  path_len, "%s/.wmwork", getenv("HOME"));
+	snprintf(logname,  path_len, "%s/worklog", dirName);
+	snprintf(lockname, path_len, "%s/.#LOCK",  dirName);
+	if (chdir(dirName) < 0) {
 		if (errno == ENOENT) {
-			if (mkdir(dirname, 0777)) {
-				fprintf(stderr, "%s: cannot mkdir '%s'\n", PACKAGE_NAME, dirname);
+			if (mkdir(dirName, 0777)) {
+				fprintf(stderr, "%s: cannot mkdir '%s'\n", PACKAGE_NAME, dirName);
 				fprintf(stderr, "%s: %s\n", PACKAGE_NAME, strerror(errno));
 				exit(1);
 			}
 			compat();
 		} else {
-			fprintf(stderr, "%s: cannot chdir into '%s'\n", PACKAGE_NAME, dirname);
+			fprintf(stderr, "%s: cannot chdir into '%s'\n", PACKAGE_NAME, dirName);
 			fprintf(stderr, "%s: %s\n", PACKAGE_NAME, strerror(errno));
 			exit(1);
 		}
@@ -302,6 +310,15 @@ void do_opts(int argc, char *argv[], int *show_days, char **xdisplay, char **geo
 		{NULL,       0, NULL,  0}};
 	int i, opt_index = 0;
 
+#ifdef SOLARIS
+	for (i = 1; i < argc; i++) {
+		if (*argv[i] == '-') {
+			if (argv[i][1] == 'd')
+				*show_days = 1;
+			else if (argv[i][1] == 'f')
+				*force = 1;
+			else if (argv[i][1] == 'h') {
+#else /* SOLARIS */
 	while (1) {
 		if ((i = getopt_long(argc, argv, "dfhv", long_opts, &opt_index)) == -1)
 			break;
@@ -313,6 +330,7 @@ void do_opts(int argc, char *argv[], int *show_days, char **xdisplay, char **geo
 			*force = 1;
 			break;
 		case 'h':
+#endif /* SOLARIS */
 			printf("usage: %s [options]\n", argv[0]);
 			printf("  -d, --days        display time in ddd.hh:mm instead of hhh:mm:ss,\n");
 			printf("  -f, --force       overwrite stale lock files,\n");
@@ -321,10 +339,28 @@ void do_opts(int argc, char *argv[], int *show_days, char **xdisplay, char **geo
 			printf("  --display=<id>    open the mini window on display <id>, e.g. ':0.0',\n");
 			printf("  --geometry=<pos>  open the mini window at position <pos>, e.g. '+10+10'.\n");
 			exit(0);
+#ifdef SOLARIS
+			} else if (argv[i][1] == 'v') {
+#else /* SOLARIS */
 		case 'v':
+#endif /* SOLARIS */
 			printf("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-			printf("copyright (c) 2002-2003 Martin A. Godisch <martin@godisch.de>\n");
+			printf("copyright (c) 2002-2005 Martin A. Godisch <martin@godisch.de>\n");
 			exit(0);
+#ifdef SOLARIS
+			} else if (!strcmp(argv[i], "--display"))
+				*xdisplay = optarg;
+			else if (!strcmp(argv[i], "--geometry"))
+				*geometry = optarg;
+			else break;
+		} else
+			break;
+	}
+	if (i < argc) {
+		fprintf(stderr, "%s: invalid argument -- %s\n", PACKAGE_NAME, argv[i]);
+		exit(1);
+	}
+#else /* SOLARIS */
 		case 0:
 			*xdisplay = optarg;
 			break;
@@ -339,6 +375,7 @@ void do_opts(int argc, char *argv[], int *show_days, char **xdisplay, char **geo
 		fprintf(stderr, "%s: invalid argument -- %s\n", PACKAGE_NAME, argv[optind]);
 		exit(1);
 	}
+#endif /* SOLARIS */
 }
 
 void drawTime(time_t time1, time_t time2, int microtm, int show_days, const int running)
@@ -431,7 +468,7 @@ void drawProject(const char *name)
 	int i;
 
 	for (i = 0; i < 3; i++) {
-		copyXPMArea(CHAR_SRC_X1(' '), CHAR_SRC_Y1, 5, 7, 23 + i * 6, 35);
+		copyXPMArea(CHAR_SRC_X2(' '), CHAR_SRC_Y2, 5, 7, 23 + i * 6, 35);
 		if (i >= strlen(name))
 			continue;
 		if (name[i] >= 'A' && name[i] <= '`')
@@ -440,8 +477,6 @@ void drawProject(const char *name)
 			copyXPMArea(CHAR_SRC_X2(name[i]), CHAR_SRC_Y2, 5, 7, 23 + i * 6, 35);
 		else if (name[i] >= '{' && name[i] <= '~')
 			copyXPMArea(CHAR_SRC_X3(name[i]), CHAR_SRC_Y3, 5, 7, 23 + i * 6, 35);
-		else
-			copyXPMArea(CHAR_SRC_X2(' '),     CHAR_SRC_Y2, 5, 7, 23 + i * 6, 35);
 	}
 }
 
@@ -618,13 +653,13 @@ int make_lock(int force)
 	if ((fd = open(lockname, O_WRONLY | O_CREAT | (force ? 0 : O_EXCL), 0666)) < 0) {
 		if (errno == EEXIST) {
 			if ((F = fopen(lockname, "r")) != NULL) {
-				i = fscanf(F, "%u", &pid);
+				i = fscanf(F, PID_T, &pid);
 				fclose(F);
-				snprintf(proc, sizeof(proc), "/proc/%u/exe", pid);
+				snprintf(proc, sizeof(proc), "/proc/"PID_T"/exe", pid);
 				if (i == 1 && ((i = readlink(proc, buffer, sizeof(buffer)-1)) >= 0 || errno == ENOENT)) {
 					buffer[i] = 0;
 					if (i < 0 || ((c = rindex(buffer, '/')) != NULL && strcmp(c+1, "wmwork") != 0)) {
-						fprintf(stderr, "%s: found stale lock file (pid %u)\n", PACKAGE_NAME, pid);
+						fprintf(stderr, "%s: found stale lock file (pid "PID_T")\n", PACKAGE_NAME, pid);
 						return make_lock(1);
 					}
 				}
@@ -637,7 +672,7 @@ int make_lock(int force)
 		return 0;
 	}
 	if ((F = fdopen(fd, "w")) != NULL) {
-		fprintf(F, "%u\n", getpid());
+		fprintf(F, PID_T"\n", getpid());
 		fclose(F);
 	} else
 		close(fd);
@@ -680,7 +715,7 @@ int compat(void)
 		if (strstr(entry->d_name, ".wmwork.") == entry->d_name &&
 			strlen(entry->d_name) > 8 && strlen(entry->d_name) <= 11) {
 			snprintf(temp1, path_len, "%s/%s", getenv("HOME"), entry->d_name);
-			snprintf(temp2, path_len, "%s/%s", dirname, entry->d_name + 8);
+			snprintf(temp2, path_len, "%s/%s", dirName, entry->d_name + 8);
 			if (rename(temp1, temp2) < 0) {
 				fprintf(stderr, "%s: cannot rename '%s' to '%s'\n", PACKAGE_NAME, temp1, temp2);
 				fprintf(stderr, "%s: %s\n", PACKAGE_NAME, strerror(errno));
